/* stylelint-disable number-max-precision, declaration-block-single-line-max-declarations, comment-empty-line-before, rule-empty-line-before */

.shift-right {
  position: relative;
  animation-name: shift-right-anim;
  animation-duration: 0.5s;
}

@keyframes shift-right-anim {
  0% {
    background-color: white;
    left: 0;
  }
  50% {
    background-color: deepskyblue;
    left: 10px;
  }
  100% {
    background-color: white;
    left: 0;
  }
}

.tight {
  margin-bottom: 0.5rem;
}

.model-wrapper {
  position: relative;
}

.modal {
  display: none; /* Hidden by default */
  position: absolute;
  z-index: 1; /* Sit on top */
  /*  left: 0;*/
  /*  top: 0;*/
  width: 100%; /* Full width */
  /*  height: 100%; /* Full height */
  overflow: auto; /* Enable scroll if needed */
  background-color: white;
  border: 2px solid #888;
}

/* Modal Content/Box */
.modal-content {
  margin: auto; /* 15% from the top and centered */
  padding: 20px;
  height: 100%; /* Full height */
  /*  width: 60%;  Could be more or less, depending on screen size */
}

/* The Close Button */
.close {
  color: #aaa;
  float: right;
  font-size: 56px;
  font-weight: bold;
}

.close:hover,
.close:focus {
  color: black;
  text-decoration: none;
  cursor: pointer;
}

.checkbox-grid li {
  display: block;
  float: left;
}

.narrower {
  color: #592a71;
}

.broader {
  color: #aa3c39;
}

.exact {
  color: #2d8632;
}

.approximate {
  color: #a8aa39;
}

.selected-element {
  box-shadow: 0 0 5px 5px #33c3f0;
}
