<!DOCTYPE html>
<html lang="en">
<head>

  <!-- Basic Page Needs
  –––––––––––––––––––––––––––––––––––––––––––––––––– -->
  <meta charset="utf-8">
  <title>{{title}}</title>
  <meta name="description" content="">
  <meta name="author" content="">

  <!-- Mobile Specific Metas
  –––––––––––––––––––––––––––––––––––––––––––––––––– -->
  <meta name="viewport" content="width=device-width, initial-scale=1">

  <!-- FONT
  –––––––––––––––––––––––––––––––––––––––––––––––––– -->
  <link href="//fonts.googleapis.com/css?family=Raleway:400,300,600" rel="stylesheet" type="text/css">

  <!-- ICONS
  –––––––––––––––––––––––––––––––––––––––––––––––––– -->
  <link rel="stylesheet" href="https://cdnjs.cloudflare.com/ajax/libs/font-awesome/6.4.0/css/all.min.css">

  <!-- CSS
  –––––––––––––––––––––––––––––––––––––––––––––––––– -->
  <link rel="stylesheet" href="{{ url_for('static', filename='css/normalize.css') }}">
  <link rel="stylesheet" href="{{ url_for('static', filename='css/tooltip.css') }}">
  <link rel="stylesheet" href="{{ url_for('static', filename='css/skeleton.css') }}">
  <link rel="stylesheet" href="{{ url_for('static', filename='css/customizations.css') }}">
  <link rel="stylesheet" href="{{ url_for('static', filename='css/layout.css') }}">

  <!-- Scripts (TODO: to move to the body end?)
  –––––––––––––––––––––––––––––––––––––––––––––––––– -->
  <script src="{{ url_for('static', filename='js/common-helpers.js') }}"></script>

  <!-- Favicon
  –––––––––––––––––––––––––––––––––––––––––––––––––– -->
  <link rel="icon" type="image/png" href="{{ url_for('static', filename='images/favicon.ico') }}">

</head>
<body>
